<?php

namespace Teknisa\MAN\Controllers;
use Zeedhi\Framework\DataSource\FilterCriteria; 
use Zeedhi\Framework\Controller\CrudWithParameterBag;
use Zeedhi\Framework\DTO;
use Doctrine\ORM\EntityManager;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Teknisa\MAN\Utilities;
use Teknisa\MAN\Services\CadBeneficioService;

class CadBeneficio extends \Teknisa\MAN\Controllers\TeknisaCrud {
    

    protected $dataSourceName = 'cadBeneficio';
    protected $parameterBagColumns = array('NRORG');
    protected $cadBeneficioService;
    
    public function __construct ( ParameterBag $parameterBag, Manager $dataSourceManager, CadBeneficioService $cadBeneficioService, Utilities\Util $utilService, EntityManager $entityManager, $crudService   ) {
        $this->cadBeneficioService = $cadBeneficioService;        
        $this->dataSourceManager = $dataSourceManager;
	    $this->parameterBag = $parameterBag;
	    $this->crudService =$crudService;
        $this->utilService = $utilService;
        $this->entityManager = $entityManager;
    }
    
    public function find(DTO\Request\Filter $request, DTO\Response $response){
        $this->dataSourceName = 'cadBeneficio';
         $conditions = $this->utilService->getParamsAsArray($request);
         $dataSourceFilter = array();
         
        if(array_key_exists('IDTPBENEFICIO',$conditions)){
         $conditions['TIDTPBENEFICIO'] = '';
        } else {
            $conditions['IDTPBENEFICIO'] = '';
        }
        
        foreach($conditions as $key => $value) {
            $dataSourceFilter[] = array(
                "columnName" => $key,
                "operator"=> "=",
                "value"=> $value
            );
        }
        
        for($i = 0; $i < count($conditions); $i++) {
            if(strpos($dataSourceFilter[$i]["columnName"], '|') != false) {
                $dataSourceFilter[$i]["operator"] = "MAPPED_LIKE_ALL";
            }
        }
        
        $newRequest = new DTO\Request\Filter(
            new FilterCriteria(
                $request->getFilterCriteria()->getDataSourceName(),
                $dataSourceFilter,
                $request->getFilterCriteria()->getPage(),
                $request->getFilterCriteria()->getPageSize()
            ),
            $request->getMethod(),
            $request->getRoutePath(),
            $request->getUserId()
        );
    	$this->parameterBagColumns = array('TIDTPBENEFICIO','IDTPBENEFICIO', 'NRORG');
    	
    	parent::find($newRequest, $response);
    }
    
    public function saveWithoutNotification(DTO\Request\DataSet $request, DTO\Response $response) {
	   	try {
            $dataSet = $request->getDataSet();
            $row = $dataSet->getRows()[0];
            if($row['__is_new']){
                $paramsCount = array(
                    'NRORG' => $row['NRORG'],
                    'NRSEQBENEFICIO' => $row['NRSEQBENEFICIO']);
                $count  = $this->entityManager->getConnection()->fetchAll(self::COUNT_NRSEQBENEFICIO, $paramsCount);
    			if($count[0]['COUNT(*)'] > 0){
    			    throw new \Exception("Código já existente!");   
    			    $error[0] = "Código já existente";
    			    $response->addDataSet(new DataSet('Error', $error));
    			}
            }
            
            $row['VRBENEFICIO']      = (float) $row['VRBENEFICIO'];
            	$params = array(
        			":IDTPATIVABENEFICIO"	=>  $row['IDTPATIVABENEFICIO'],
        			":QTDMAXDIASUBSEQ"		=>	$row['QTDMAXDIASUBSEQ'],
        			":VRBENEFICIO"			=>	$row['VRBENEFICIO'],
        			":NRORG" 				=>  $row['NRORG'],
        			":DSBENEFICIO"			=>	$row['DSBENEFICIO'],
        			":IDTPBENEFICIO"		=>	$row['IDTPBENEFICIO'],
        			":QTDMAXDIACOMPRA"		=>	$row['QTDMAXDIACOMPRA'],
        			":NRSEQBENEFICIO"		=>	$row['NRSEQBENEFICIO']
        		);
            $dataSet->setDataSourceName($this->dataSourceName);
            $retorno = $this->cadBeneficioService->saveBeneficio($row['__is_new'], $params);
            $response->addDataSet(new DataSet($this->dataSourceName, array()));
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        }
	}
	
	public function deleteWithoutNotification (DTO\Request\Row $request,DTO\Response $response) {
		try {
		    $this->dataSourceName = 'cadBeneficio';
            parent::delete($request,$response);
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        }
	}
	
	public function criaCodigoBeneficio (DTO\Request $request, DTO\Response $response){
	        $params = $this->utilService->getParamsAsArray($request);
	        $seqcode[0] = $this->getNewSeqCode(10, $params['NRORG']);
	        $response->addDataSet(new DataSet('CodigoBeneficio', $seqcode));
	}
	
	private function getNewSeqCode ($qtdString, $nrorg) {
		$params = array("NRORG" => $nrorg);
		$seqcode = $this->entityManager->getConnection()->fetchAll(self::GET_MAX_NRBENEFICIO, $params)[0]['MAX'];
		$seqcode++;
		$seqcode = str_pad($seqcode, $qtdString, "0", STR_PAD_LEFT);
		return $seqcode;
	}
	
	const GET_MAX_NRBENEFICIO = "
	    SELECT MAX(NRSEQBENEFICIO) AS MAX
	        FROM BENEFICIOFOS 
	        WHERE NRORG = :NRORG
	    ";
	    
	const COUNT_NRSEQBENEFICIO = "
	SELECT COUNT(*)
	    FROM BENEFICIOFOS
	    WHERE NRORG = :NRORG
	    AND NRSEQBENEFICIO = :NRSEQBENEFICIO";
}
